import { computed, isRef } from "vue";
import { useAsyncData } from "#imports";
import { hash } from "ohash";
export function useFetch(request, opts = {}) {
  if (process.dev && !opts.key && Object.values(opts).some((v) => typeof v === "function" || v instanceof Blob)) {
    console.warn("[nuxt] [useFetch] You should provide a key when passing options that are not serializable to JSON:", opts);
  }
  if (!request) {
    throw new Error("[nuxt] [useFetch] request is missing.");
  }
  const key = "$f_" + (opts.key || hash([request, { ...opts, transform: null }]));
  const _request = computed(() => {
    let r = request;
    if (typeof r === "function") {
      r = r();
    }
    return isRef(r) ? r.value : r;
  });
  const { server, lazy, default: defaultFn, transform, pick, watch, initialCache, ...fetchOptions } = opts;
  const _fetchOptions = {
    ...fetchOptions,
    cache: typeof opts.cache === "boolean" ? void 0 : opts.cache
  };
  const _asyncDataOptions = {
    server,
    lazy,
    default: defaultFn,
    transform,
    pick,
    initialCache,
    watch: [
      _request,
      ...watch || []
    ]
  };
  const asyncData = useAsyncData(key, () => {
    const method = opts && opts.method && ["get", "head", "delete", "post", "put", "patch"].includes(opts.method.toLowerCase()) ? opts.method.toLowerCase() : "get";
    return globalThis.$fetch["$" + method](_request.value, _fetchOptions);
  }, _asyncDataOptions);
  return asyncData;
}
export function useLazyFetch(request, opts = {}) {
  return useFetch(request, {
    ...opts,
    lazy: true
  });
}
export function useHttp(request, opts = {}) {
  if (process.dev && !opts.key && Object.values(opts).some((v) => typeof v === "function" || v instanceof Blob)) {
    console.warn("[nuxt] [useFetch] You should provide a key when passing options that are not serializable to JSON:", opts);
  }
  if (!request) {
    throw new Error("[nuxt] [useHttp] request is missing.");
  }
  const key = "$h_" + (opts.key || hash([request, { ...opts, transform: null }]));
  const _request = computed(() => {
    let r = request;
    if (typeof r === "function") {
      r = r();
    }
    return isRef(r) ? r.value : r;
  });
  const { server, lazy, default: defaultFn, transform, pick, watch, initialCache, ...fetchOptions } = opts;
  const _fetchOptions = {
    ...fetchOptions,
    cache: typeof opts.cache === "boolean" ? void 0 : opts.cache
  };
  const _asyncDataOptions = {
    server,
    lazy,
    default: defaultFn,
    transform,
    pick,
    initialCache,
    watch: [
      _request,
      ...watch || []
    ]
  };
  const asyncData = useAsyncData(key, () => {
    const method = opts && opts.method && ["get", "head", "delete", "post", "put", "patch"].includes(opts.method.toLowerCase()) ? opts.method.toLowerCase() : "get";
    return globalThis.$http["$" + method](_request.value, _fetchOptions);
  }, _asyncDataOptions);
  return asyncData;
}
export function useLazyHttp(request, opts = {}) {
  return useHttp(request, {
    ...opts,
    lazy: true
  });
}
